@@ -1,5 +1,5 @@
 # Scalpel sound editor (http://scalpelsound.online.fr)
-# Copyright 2008 (C) Pierre Duquesne <stackp@online.fr>
+# Copyright 2009 (C) Pierre Duquesne <stackp@online.fr>
 # Licensed under the Revised BSD License.
 
 from event import Signal
@@ -63,19 +63,21 @@ class History(object):
 
 class Sound(object):
 
-    # _data is a numpy.ndarray, as returned by pysndfile
+    # frames is a numpy.ndarray, as returned by pysndfile
     
     def __init__(self, filename=None):
         self.filename = filename
         if filename == None:
             # empty sound
-            self._data = numpy.array([])
+            self.frames = numpy.array([])
             self.numchan = 1
+            self.samplerate = 44100
         else:
             f = pysndfile.sndfile(filename)
             nframes = f.get_nframes()
-            self._data = f.read_frames(nframes)
+            self.frames = f.read_frames(nframes)
             self.numchan = f.get_channels()
+            self.samplerate = f.get_samplerate()
             f.close()
         self.history = History()
         self.changed = Signal()
@@ -88,47 +90,38 @@ class Sound(object):
                               format=format,
                               channels=self.numchan,
                               samplerate=44100)
-        f.write_frames(self._data)
+        f.write_frames(self.frames)
         f.close()
         self.filename = filename
 
     def cut(self, start, end):
-        clip = self._data[start:end]
+        clip = self.frames[start:end]
         do = (self._do_cut, (start, end))
-        undo = (self._do_paste, (start, self._data[start:end]))
+        undo = (self._do_paste, (start, copy(self.frames[start:end])))
         self.history.add(do, undo)
+        self.changed()
         return clip
     
     def _do_cut(self, start, end):
-        data = numpy.concatenate((self._data[:start], self._data[end:]))
-        self._data = data
-        self.changed()
+        data = numpy.concatenate((self.frames[:start], self.frames[end:]))
+        self.frames = data
 
     def copy(self, start, end):
-        clip = self._data[start:end]
+        clip = self.frames[start:end]
         return clip
 
     def paste(self, start, clip):
         do = (self._do_paste, (start, clip))
         undo = (self._do_cut, (start, start + len(clip)))
         self.history.add(do, undo)
+        self.changed()
 
     def _do_paste(self, start, clip):
-        data = numpy.concatenate((self._data[:start], clip,self._data[start:]))
-        self._data = data
-        self.changed()
-        
-    def normalize(self, start, end):
-        pass
+        data = numpy.concatenate((self.frames[:start], clip,self.frames[start:]))
+        self.frames = data
 
-    def reverse(self, start, end):
-        do = (self._do_reverse, (start, end))
-        undo = (self._do_reverse, (start, end))
-        self.history.add(do, undo)
-
-    def _do_reverse(self, start, end):
-        rev = numpy.flipud(copy(self._data[start:end]))
-        self._data[start:end] = rev
+    def apply(self, fx):
+        self.history.add((fx.apply, ()), (fx.revert, ()))
         self.changed()
 
     def monoize(self, start, end):
@@ -136,9 +129,15 @@ class Sound(object):
     
     def undo(self):
         self.history.undo()
+        self.changed()
 
     def redo(self):
         self.history.redo()
+        self.changed()
+
+    def is_fresh(self):
+        """True if sound is empty and has never been edited."""
+        return not len(self.frames) and not self.history._actions
 
 
 # -- Tests
@@ -189,84 +188,73 @@ def testSound():
     sine = [sin(2 * 3.14 * f0/SR * x) for x in range(time * SR)]
     start, end = 2*SR, 4*SR
     sine2 = copy(sine)
-    snd._data = copy(sine)
+    snd.frames = copy(sine)
     del sine2[start:end]
     snd.cut(start, end)
-    assert snd._data.tolist() == sine2
+    assert snd.frames.tolist() == sine2
 
     snd.history.undo()
-    assert snd._data.tolist() == sine
+    assert snd.frames.tolist() == sine
     snd.history.undo()
-    assert snd._data.tolist() == sine
+    assert snd.frames.tolist() == sine
     snd.history.redo()
-    assert snd._data.tolist() == sine2
+    assert snd.frames.tolist() == sine2
 
     clip = snd.copy(start, end)
     snd.paste(0, clip)
-    assert snd._data.tolist() == sine2[start:end] + sine2
+    assert snd.frames.tolist() == sine2[start:end] + sine2
     snd.history.undo()
-    assert snd._data.tolist() == sine2
+    assert snd.frames.tolist() == sine2
     snd.history.redo()
-    assert snd._data.tolist() == sine2[start:end] + sine2
+    assert snd.frames.tolist() == sine2[start:end] + sine2
     
     # test with a mono file
     snd = Sound("../sounds/test1.wav")
-    assert snd._data != []
+    assert snd.frames != []
     start = 4444
     end = 55555
-    data = snd._data.tolist()
-    data2 = snd._data.tolist()
+    data = snd.frames.tolist()
+    data2 = snd.frames.tolist()
     del data2[start:end]
     snd.cut(start, end)
-    assert snd._data.tolist() == data2
+    assert snd.frames.tolist() == data2
     snd.history.undo()
-    assert snd._data.tolist() == data
+    assert snd.frames.tolist() == data
     snd.history.undo()
-    assert snd._data.tolist() == data
+    assert snd.frames.tolist() == data
     snd.history.redo()
-    assert snd._data.tolist() == data2
+    assert snd.frames.tolist() == data2
     clip = snd.copy(start, end)
     snd.paste(0, clip)
-    assert snd._data.tolist() == data2[start:end] + data2
+    assert snd.frames.tolist() == data2[start:end] + data2
     snd.history.undo()
-    assert snd._data.tolist() == data2
+    assert snd.frames.tolist() == data2
     snd.paste(0, clip)
-    assert snd._data.tolist() == data2[start:end] + data2
+    assert snd.frames.tolist() == data2[start:end] + data2
 
     # test with a stereo file
     snd = Sound("../sounds/test2.wav")
-    assert snd._data.tolist() != []
+    assert snd.frames.tolist() != []
     start = 4444
     end = 55555
-    data = copy(snd._data.tolist())
-    data2 = copy(snd._data.tolist())
+    data = copy(snd.frames.tolist())
+    data2 = copy(snd.frames.tolist())
     del data2[start:end]
     snd.cut(start, end)
-    assert snd._data.tolist() == data2
+    assert snd.frames.tolist() == data2
     snd.history.undo()
-    assert snd._data.tolist() == data
+    assert snd.frames.tolist() == data
     snd.history.undo()
-    assert snd._data.tolist() == data
+    assert snd.frames.tolist() == data
     snd.history.redo()
-    assert snd._data.tolist() == data2
+    assert snd.frames.tolist() == data2
     clip = snd.copy(start, end)
     snd.paste(0, clip)
-    assert snd._data.tolist() == data2[start:end] + data2
+    assert snd.frames.tolist() == data2[start:end] + data2
     snd.history.undo()
-    assert snd._data.tolist() == data2
+    assert snd.frames.tolist() == data2
     snd.paste(0, clip)
-    assert snd._data.tolist() == data2[start:end] + data2
-
-    # test reverse()
-    snd = Sound()
-    snd._data = numpy.array(range(10))
-    snd.reverse(3, 6)
-    assert snd._data.tolist() == [0, 1, 2, 5, 4, 3, 6, 7, 8, 9]
-
-    snd._data = numpy.array(zip(range(8), range(8)))
-    snd.reverse(3, 6)
-    assert snd._data.tolist() == [[0, 0], [1, 1], [2, 2], [5, 5],
-                                  [4, 4], [3, 3], [6, 6], [7, 7]]
+    assert snd.frames.tolist() == data2[start:end] + data2
 
     # test save_as()
     import os
@@ -276,7 +264,7 @@ def testSound():
     assert os.path.exists(outfile)
     assert snd.filename == outfile
     snd2 = Sound(outfile)
-    assert abs((snd._data - snd2._data).max()) < 0.0001 # quantization errors!
+    assert abs((snd.frames - snd2.frames).max()) < 0.0001 # quantization errors!
     os.remove(outfile)
 
     snd = Sound("../sounds/test2.wav")
@@ -285,8 +273,8 @@ def testSound():
     assert os.path.exists(outfile)
     assert snd.filename == outfile
     snd2 = Sound(outfile)
-    assert snd._data.size == snd2._data.size
-    assert abs((snd._data - snd2._data).flatten().max()) == 0
+    assert snd.frames.size == snd2.frames.size
+    assert abs((snd.frames - snd2.frames).flatten().max()) == 0
     os.remove(outfile)
 
         
